---
title: {{ story.name }}
---
{% if story.info['experimental'] %}
!!! warning "Experimental"

    This feature is in alpha. The API may change on a minor version increment.
{% endif %}

{{ story.info['description'] }}

{% if 'yaml_snippet' in story.given %}Example yaml_snippet:

```yaml
{{ story.given['yaml_snippet'] }}
```
{% endif %}

{% if 'setup' in story.given -%}
```python
{{ story.given['setup'] }}
```
{%- endif %}

{% if story.variations %}
{% for variation in story.variations %}
{{ variation.child_name }}:
{% if'yaml_snippet' in variation.data.get('given', {}).keys() %}
```yaml
{{ variation.given['yaml_snippet'] }}
```
{% endif %}
{% with step = variation.steps[0] %}{% include "step.jinja2" %}{% endwith %}
{% endfor %}
{% else %}
{% with step = story.steps[0] %}{% include "step.jinja2" %}{% endwith %}
{% endif %}


!!! note "Executable specification"

    Documentation automatically generated from 
    <a href="https://github.com/crdoconnor/strictyaml/blob/master/hitch/story/{{ story.filename.basename() }}">{{ story.filename.basename() }}
    storytests.
